package com.zyh.es.mq;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.rabbitmq.client.Channel;
import com.zyh.common.constants.RabbitMQQueueNameConstants;
import com.zyh.common.domain.CompreFaceApiResponse;
import com.zyh.es.service.impl.EsServicempl;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.HashMap;

@Component
@Log4j2
public class Clicknum {
    @Autowired
    private EsServicempl esServicempl;
    @Autowired
    RedisTemplate<String, String> redisTemplate;

    @RabbitListener(queuesToDeclare = {@Queue(RabbitMQQueueNameConstants.SEND_SMS_QUEUE_NAME)})
    public void SendCodeMq(String jsonString, Message message, Channel channel) throws JsonProcessingException {
        long l = System.currentTimeMillis();
        log.info("短信消费者队列开始消费...，消费内容为：{}", jsonString);
        String messageId = message.getMessageProperties().getMessageId();

        ObjectMapper objectMapper = new ObjectMapper();
        CompreFaceApiResponse.Result result = objectMapper.readValue(jsonString, CompreFaceApiResponse.Result.class);

        esServicempl.pushToElasticsearch(result);

        try {
            Long count = redisTemplate.opsForSet().add(RabbitMQQueueNameConstants.SEND_SMS_QUEUE_NAME, messageId);
            if (count == 1) {

                  log.info("短信消费者队列消费成功，耗时：{}毫秒！", System.currentTimeMillis()-l);
            }else{
                log.info("短信消费者队列重复消费，消费内容为：{}", jsonString);
            }

        } catch (Exception e) {
            log.info("短信消费者队列消费失败，异常内容为：{}", e.getMessage());
            redisTemplate.opsForSet().remove(RabbitMQQueueNameConstants.SEND_SMS_QUEUE_NAME, messageId);

            try {
                channel.basicReject(message.getMessageProperties().getDeliveryTag(),true);
            } catch (Exception ex) {
                log.info("短信消费者队列回退失败，异常内容为：{}", ex.getMessage());
            }
        }
    }
}
