package com.zyh.system.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.nacos.common.utils.CollectionUtils;
import com.zyh.common.domain.House;
import com.zyh.common.domain.Person;
import com.zyh.common.domain.PersonTest;
import com.zyh.common.result.Result;
import com.zyh.common.util.ImportExeclUtil;
import com.zyh.common.utils.ExcelUtil;
import com.zyh.common.utils.LyException;
import com.zyh.common.utils.StringUtils;
import com.zyh.system.service.PersonService;
import lombok.extern.log4j.Log4j2;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

@RestController
@RequestMapping("person")
@Log4j2
public class PersonController {
    @Autowired
    private PersonService personService;

    //新增人口
    @PostMapping("addPerson")
    public Result addPerson(@RequestBody Person person) {
        return personService.addPerson(person);
    }

    //查询人口
    @PostMapping("getPerson")
    public Result<List<Person>> getPerson() {
        return personService.getPerson();
    }

    //删除人口
    @PostMapping("deletePerson")
    public Result deletePerson(@RequestParam Integer id, @RequestParam String buildId, @RequestParam Integer unit, @RequestParam String holdId) {
        return personService.deletePerson(id, buildId, unit, holdId);
    }

    //楼栋下拉框
    @GetMapping("getBuildId")
    public Result<List<Integer>> getBuildId() {
        return personService.getBuildId();
    }

    //单元下拉框
    @GetMapping("getUnitId")
    public Result<List<Integer>> getUnitId(@RequestParam Integer buildId) {
        return personService.getUnitId(buildId);
    }

    //房间号下拉框
    @GetMapping("getHoldId")
    public Result<List<Integer>> getHoldId(@RequestParam Integer buildId, @RequestParam Integer unit) {
        return personService.getHoldId(buildId, unit);
    }

    //小区下拉框
    @GetMapping("getHouseId")
    public Result<List<House>> getHouseId() {
        return personService.getHouseId();
    }

    //修改人口
    @PostMapping("updatePerson")
    public Result updatePerson(@RequestBody Person person) {
        return personService.updatePerson(person);
    }




    @GetMapping(value = "/importPersonInfo")
    public Result uuApplyUserInfo(@RequestParam(value = "file",required = false) MultipartFile file) {
        try {
            //工具类
            ImportExeclUtil readExcelUtil = new ImportExeclUtil();
            List<List<String>> read = readExcelUtil.read(file.getInputStream(), true);

            if (CollectionUtils.isNotEmpty(read)){

                List<PersonTest> importList = read.stream().map(e -> {
                    PersonTest importDto = new PersonTest();
                    importDto.setId(StringUtils.isNotEmpty(e.get(0)) ? Integer.valueOf(e.get(0)) : null);
                    importDto.setName(e.get(1));
                    importDto.setIdcard(e.get(2));
                    importDto.setIdcardType(StringUtils.isNotEmpty(e.get(3)) ? Integer.valueOf(e.get(3)) : null);
                    importDto.setBirth(e.get(4));
                    importDto.setNation(e.get(5));
                    importDto.setSex(StringUtils.isNotEmpty(e.get(6)) ? Integer.valueOf(e.get(6)) : null);
                    importDto.setPhone(e.get(7));
                    importDto.setAddress(e.get(8));
                    importDto.setHouseId(e.get(9));
                    importDto.setBuildId(StringUtils.isNotEmpty(e.get(10)) ? Integer.valueOf(e.get(10)) : null);
                    importDto.setUnit(StringUtils.isNotEmpty(e.get(11)) ? Integer.valueOf(e.get(11)) : null);
                    importDto.setHoldId(StringUtils.isNotEmpty(e.get(12)) ? Integer.valueOf(e.get(12)) : null);
                    importDto.setPic(e.get(13));

                    return importDto;
                }).collect(Collectors.toList());

                if (CollectionUtils.isEmpty(importList)){
                    return Result.error("不能导入空文件");
                }

                //最多导入1W条
                final int maxInt = 10000;
                if (importList.size() > maxInt){
                    return Result.error("导入最多修改1W条");
                }

                    Result result = personService.uuApplyUserInfo(importList);

                    return result;

            }else{
                return Result.error("不能导入空文件");
            }

        } catch (Exception e) {
            e.printStackTrace();
            return Result.error("导入失败，更新数据库时报错！报错信息：" + e.toString());
        }
    }







    @PostMapping("derive")
    public String derive() throws IOException {
        return personService.derive();
    }



}
