package com.zyh.system.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.zyh.common.Enum.WriteRowsOption;
import com.zyh.common.domain.House;
import com.zyh.common.domain.Person;
import com.zyh.common.domain.PersonTest;
import com.zyh.common.result.Result;
import com.zyh.common.util.ImportExeclUtil;
import com.zyh.common.utils.StringUtils;
import com.zyh.system.mapper.PersonMapper;
import com.zyh.system.service.PersonService;
import lombok.extern.log4j.Log4j2;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

@Service
@Log4j2
public class PersonServicempl implements PersonService {
    @Autowired
    private PersonMapper personMapper;
    @Autowired
    private HttpServletResponse response;
    @Override
    public Result addPerson(Person person) {
        Integer add = personMapper.addPerson(person);
        if(add>0){
            //修改房间人数
            personMapper.updatepersonNumjia(person.getBuildId(), person.getUnit(), person.getHoldId());
            return Result.success("新增成功");
        }

        return Result.error("新增失败");
    }

    @Override
    public Result<List<Person>> getPerson() {
        List<Person> person = personMapper.getPerson();
        return Result.success(person);
    }

    @Override
    public Result deletePerson(Integer id, String buildId, Integer unit, String holdId) {
        Integer delete = personMapper.deletePerson(id);
        if(delete>0){
            //修改房间人数
            personMapper.updatepersonNum(buildId, unit, holdId);
            return Result.success("删除成功");
        }
        return Result.error("删除失败");
    }

    @Override
    public Result<List<Integer>> getBuildId() {
        List<Integer> buildId = personMapper.getBuildId();
        return Result.success(buildId);
    }

    @Override
    public Result<List<Integer>> getUnitId(Integer buildId) {
        List<Integer> unitId = personMapper.getUnitId(buildId);
        return Result.success(unitId);
    }

    @Override
    public Result<List<Integer>> getHoldId(Integer buildId, Integer unit) {
        List<Integer> holdId = personMapper.getHoldId(buildId, unit);
        return Result.success(holdId);
    }

    @Override
    public Result<List<House>> getHouseId() {
        List<House> houseId = personMapper.getHouseId();
        return Result.success(houseId);
    }

    @Override
    public Result updatePerson(Person person) {
        Integer update = personMapper.updatePerson(person);
        if(update>0){
            return Result.success("更新成功");
        }
        return Result.error("更新失败");
    }

    @Override
    public Result uuApplyUserInfo(List<PersonTest> importList) {
        int i = 0;
        List<String> importResultColumn = new ArrayList<>(); // 新列，用于存储导入结果
        List<String> resultList = createResultList(importList);
        if (resultList.size() == 0) {
            for (PersonTest personTest : importList) {
                try {
                    Integer add = personMapper.addPersonTest(personTest);

                    // 修改房间人数
                    personMapper.updatepersonNumjia(personTest.getBuildId(), personTest.getUnit(), personTest.getHoldId());
                    i++;

                    log.info("第" + i + "条导入成功");
                    importResultColumn.add("导入成功");
                } catch (Exception e) {
                    i++;
                    log.error("第" + i + "条导入失败，失败原因：" + e.getMessage());
                    importResultColumn.add("导入失败，原因：" + e.getMessage());
                }
            }
        }else{
            for (String s : resultList) {
                importResultColumn.add(s);
            }
        }


        // 向原始Excel文件添加新列
        updateOriginalExcel(importList, importResultColumn);

        return Result.success("导入完毕");
    }

    private void updateOriginalExcel(List<PersonTest> importList, List<String> importResultColumn) {
        String fileName = "PersonTest";
        try {
            // 加载原始Excel文件
            ImportExeclUtil readExcelUtil = new ImportExeclUtil();
            List<List<String>> originalData = readExcelUtil.read(new FileInputStream("D:/2104A/daochu/" + fileName + ".xlsx"), true);



            // 将导入结果值添加到每一行
            for (int i = 0; i < importList.size(); i++) {
                originalData.get(i).add(importResultColumn.get(i)); // 注意索引的调整
            }

            // 将更新后的数据写回原始Excel文件
            writeDataToOriginalExcel(originalData, "D:/2104A/daochu/" + fileName + ".xlsx");
        } catch (IOException e) {
            log.error("更新原始Excel文件时出错：" + e.getMessage());
        }
    }

    private void writeDataToOriginalExcel(List<List<String>> data, String filePath) {
        try {
            Workbook wb = ImportExeclUtil.chooseWorkbook(filePath, new FileInputStream(filePath));
            Sheet sheet = wb.getSheetAt(0);

            // 清空第一行以下的所有行，保留第一行标题
            for (int i = sheet.getLastRowNum(); i > 0; i--) {
                Row row = sheet.getRow(i);
                if (row != null) {
                    sheet.removeRow(row);
                }
            }

            // 将数据写入表格，从第二行开始
            for (int i = 0; i < data.size(); i++) {
                Row row = sheet.createRow(i + 1);  // 注意这里是 i + 1
                for (int j = 0; j < data.get(i).size(); j++) {
                    Cell cell = row.createCell(j);

                    // 将空值用空字符串代替，你也可以使用其他默认值
                    String cellValue = data.get(i).get(j) != null ? data.get(i).get(j) : "";
                    cell.setCellValue(cellValue);
                }
            }

            // 将更改写回文件
            try (FileOutputStream fileOut = new FileOutputStream(filePath)) {
                wb.write(fileOut);
            }

            // 关闭工作簿
            wb.close();
        } catch (IOException e) {
            log.error("将数据写入原始Excel文件时出错：" + e.getMessage());
        }
    }



    @Override
    public List<Person> searchAdmin(Person person) {
        return personMapper.searchAdmin(person);
    }

    @Override
    public String derive() throws IOException {
        download();
        return null;
    }





    public void download() throws IOException {
        log.info("*********导出开始！**************");
        long startTime = System.currentTimeMillis();
        String fileName = "PersonTest";
        OutputStream outputStream = null;

        try {
            int totalCount = WriteRowsOption.TOTAL_COUNT;  // 总行数
            int sheetDataRows = WriteRowsOption.SHEET_DATA_ROWS;  // 每个 Sheet 的行数
            int writeDataRows = WriteRowsOption.WRITE_DATA_ROWS;  // 每次写入的行数

            // 获取路径
            String filePath = "D:/2104A/daochu/" + fileName + ".xlsx";

            // 计算需要的 Sheet 数量
            Integer sheetNum = totalCount % sheetDataRows == 0 ? (totalCount / sheetDataRows) : (totalCount / sheetDataRows + 1);
            // 计算一般情况下每一个 Sheet 需要写入的次数
            Integer oneSheetWriteCount = sheetDataRows / writeDataRows;
            // 计算最后一个 Sheet 需要写入的次数
            Integer lastSheetWriteCount = totalCount % sheetDataRows == 0 ?
                    oneSheetWriteCount :
                    (totalCount % sheetDataRows % writeDataRows == 0 ?
                            (totalCount / sheetDataRows / writeDataRows) :
                            (totalCount / sheetDataRows / writeDataRows + 1));
            outputStream = new FileOutputStream(filePath);
            ExcelWriter excelWriter = EasyExcel.write(outputStream).build();

            for (int i = 0; i < sheetNum; i++) {
                // 创建 Sheet
                WriteSheet sheet = new WriteSheet();
                sheet.setSheetName("Sheet" + i);
                sheet.setSheetNo(i);

                for (int j = 0; j < (i != sheetNum - 1 ? oneSheetWriteCount : lastSheetWriteCount); j++) {
                    // 创建 WriteSheet
                    WriteSheet writeSheet = EasyExcel.writerSheet(i, "Sheet" + (i + 1)).head(PersonTest.class)
                            .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy()).build();

                    // 从数据库中提取数据
                    List<PersonTest> data = fetchDataFromDatabase(i * sheetDataRows + j * writeDataRows, writeDataRows);
                    // 将数据写入 Excel
                    excelWriter.write(data, writeSheet);
                }
            }

            long endTime = System.currentTimeMillis();
            long elapsedTime = endTime - startTime;
            log.info("*********导出结束！导出耗时：" + elapsedTime + "毫秒**************");

            // 关闭 ExcelWriter
            excelWriter.finish();
            outputStream.flush();
            outputStream.close();

            // 返回给前端的响应
            response.setContentType("application/octet-stream");
            response.setCharacterEncoding("utf-8");
            response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");

            // 从生成的文件读取内容并写入响应输出流
            try (InputStream inputStream = new FileInputStream(filePath);
                 OutputStream responseOutputStream = response.getOutputStream()) {
                byte[] buffer = new byte[1024];
                int bytesRead;
                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    responseOutputStream.write(buffer, 0, bytesRead);
                }
            }

            // 删除生成的文件
//            File file = new File(filePath);
//            if (file.exists()) {
//                file.delete();
//            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (outputStream != null) {
                outputStream.close();
            }
        }
    }

    /**
     * 从数据库中获取数据
     *
     * @param offset 偏移量
     * @param limit  限制数量
     * @return 数据列表
     */
    public List<PersonTest> fetchDataFromDatabase(int offset, int limit) {
        // 调用数据库查询方法以检索数据，注意在 SQL 查询中使用 offset 和 limit
        return personMapper.stuListWithLimit(offset, limit);
    }


    private List<String> createResultList(List<PersonTest> importList) {
        List<String> resultList = new ArrayList<>();
        for (PersonTest personTest : importList) {
            StringBuilder ret=new StringBuilder();
            //姓名验证
            if (StringUtils.isEmpty(personTest.getName())) {
                ret.append("姓名不能为空"+"  ");
            }

            //身份证号验证
            if (StringUtils.isEmpty(personTest.getIdcard())) {
                ret.append("身份证号不能为空" + "  ");
            }else{
                if (!idcardRule(personTest.getIdcard())) {
                    ret.append("身份证号格式有误" + "  ");
                }

            }

            //身份证号编号验证
            if (personTest.getIdcardType()==null) {
                ret.append("身份证类型编号不能为空" + "  ");
            }else {
                if (personTest.getIdcardType() != 1 && personTest.getIdcardType() != 2 && personTest.getIdcardType() != 3 && personTest.getIdcardType() != 4) {
                    ret.append("身份证类型编号必须1-4数字（1居民身份证 2军官证 3警官证 4普通护照）" + "  ");

                }
            }


            //出生日期验证
            if (StringUtils.isEmpty(personTest.getBirth())) {
                ret.append("出生日期不能为空" + "  ");
            }

            //民族验证
            if (StringUtils.isEmpty(personTest.getNation())) {
                ret.append("民族不能为空" + "  ");
            }

            //性别验证
            if (personTest.getSex()==null) {
                ret.append("性别不能为空" + "  ");
            }else{
                if (personTest.getSex() != 1 && personTest.getSex() != 2 && personTest.getSex() != 3) {
                    ret.append("性别必须1-3数字（1男2女3未知）" + "  ");
                }
            }


            //手机号验证
            if (StringUtils.isEmpty(personTest.getPhone())) {
                ret.append("手机号不能为空" + "  ");
            }else{
                if (!telRule(personTest.getPhone())) {
                    ret.append("手机号格式有误" + "  ");
                }
            }


            //户籍验证
            if (StringUtils.isEmpty(personTest.getAddress())) {
                ret.append("地址不能为空" + "  ");
            }

            //小区编号验证
            if (StringUtils.isEmpty(personTest.getHouseId())) {
                ret.append("小区编号不能为空" + "  ");
            }else{
                int re=  personMapper.selectHouseId(personTest.getHouseId());
                if (re<=0) {
                    ret.append("小区编号不存在" + "  ");
                }
            }

            //楼栋编号验证
            if (personTest.getBuildId()==null) {
                ret.append("楼栋编号不能为空" + "  ");
            }else{
                Integer buildId = personMapper.selectBuildId(personTest.getBuildId());
                if (buildId<=0 || buildId==null) {
                    ret.append("楼栋编号不存在" + "  ");
                }
            }

            //单元编号验证
            if (personTest.getUnit()==null) {
                ret.append("单元编号不能为空" + "  ");
            }else{
                if (personTest.getBuildId() != null) {
                    Integer unit = personMapper.selectUnit(personTest.getBuildId(),personTest.getUnit());
                    if (unit<=0 || unit==null) {
                        ret.append("单元编号不存在" + "  ");
                    }
                }

            }

            //房号编号验证
            if (personTest.getHoldId()==null) {
                ret.append("房号编号不能为空" + "  ");
            }else{
                if (personTest.getBuildId() != null && personTest.getUnit() != null) {
                    Integer holdId = personMapper.selectHoldId(personTest.getBuildId(),personTest.getUnit(),personTest.getHoldId());
                    if (holdId<=0 || holdId==null) {
                        ret.append("房号编号不存在" + "  ");
                    }
                }

            }

            //身份证照片验证
            if (StringUtils.isEmpty(personTest.getPic())) {
                ret.append("身份证照片不能为空" + "  ");
            }

            resultList.add(ret.toString());
        }
        return resultList;
    }
    private boolean telRule(String phone) {
        Pattern compile = Pattern.compile("^(?:(?:\\+|00)86)?1(?:(?:3[\\d])|(?:4[5-7|9])|(?:5[0-3|5-9])|(?:6[5-7])|(?:7[0-8])|(?:8[\\d])|(?:9[1|8|9]))\\d{8}$");

        return compile.matcher(phone).matches();
    }

    private boolean idcardRule(String idcard) {
        Pattern compile = Pattern.compile("^[1-9]\\\\d{5}(18|19|20)\\\\d{2}(0[1-9]|1[0-2])(0[1-9]|[12]\\\\d|3[01])\\\\d{3}(\\\\d|X|x)$");

        return compile.matcher(idcard).matches();
    }

}
